
//-----------------------------------------------------------------------------
// File: tessdemo.cpp
//
// Desc: Demo of the projective mesh concept
//
// Copyright (c) 2003 Claes Johanson
//-----------------------------------------------------------------------------

#include "Shader_Water.h"

dxmouse *mouse;
float mouseX=0, mouseY=0;
float mspeed = 0.005;
bool drag=false;
bool text=true;

parameterhandler *prm;
surface *sea;

bool	keys[256];			// Array Used For The Keyboard Routine

#define PI	3.1415926535898
#define		n_cameras 2
int			active_camera_view = 0, active_camera_mouse = 0;
camera		*camera_mouse, *camera_view, *cameras[n_cameras];

extern int gridsize_x, gridsize_y;
#ifdef CPU_NORMALS
//int gridsize_x = 256, gridsize_y = 512;		
#else
//int gridsize_x = 128, gridsize_y = 256;		
#endif

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D9             g_pD3D				= NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice		= NULL; // Our rendering device
LPD3DXFONT				g_pd3dxFont			= NULL;

D3DLIGHT9 sun;

Shader_Water water;
// declarations

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
	HRESULT hr;
	// Create the D3D object.
	if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
		return E_FAIL;

	// Set m_vecUp the structure used to create the D3DDevice
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed = TRUE;
	//d3dpp.Windowed = FALSE;

	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	//d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;
	//d3dpp.BackBufferFormat = D3DFMT_A2R10G10B10;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
	//d3dpp.MultiSampleType = D3DMULTISAMPLE_4_SAMPLES;

	// Create the D3DDevice
	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
		D3DCREATE_HARDWARE_VERTEXPROCESSING,
		&d3dpp, &g_pd3dDevice ) ) )
	{
		return E_FAIL;
	}

	// initialize camera objects
	cameras[0] = new camera(D3DXVECTOR3(-27,8,0),0.3,450,0,45,1.33,0.3,5000.0);
	cameras[1] = new camera(D3DXVECTOR3(-150,50,-100),-0.5,-1.3,0,45,1.33,0.1,10000.0);
	camera_mouse = cameras[active_camera_mouse];
	camera_view = cameras[active_camera_view];

	// Create a font for statistics and help output
//	createFont();
	D3DXVECTOR3 waterpos = D3DXVECTOR3(0,0,0);
	water.Init(g_pd3dDevice, camera_view, &waterpos );

	sea = water.GetSurface();
	prm = water.GetParameta();
	return S_OK;
}

void loadpreset(int n)
{
	switch(n)
	{
	case 1:
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	0.9f );
		prm->set_float( p_fFalloff,		0.607f );
		prm->set_float( p_fWaterColourR,	0.07f );
		prm->set_float( p_fWaterColourG,	0.11f );
		prm->set_float( p_fWaterColourB,	0.11f );
		prm->set_float(	p_fLODbias,		0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	case 2:
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	4.0f );
		prm->set_float( p_fFalloff,		0.47f );
		prm->set_float( p_fWaterColourR,	0.13f );
		prm->set_float( p_fWaterColourG,	0.19f );
		prm->set_float( p_fWaterColourB,	0.22f );
		prm->set_float(	p_fLODbias,		4.5f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	case 3:
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	7.0f );
		prm->set_float( p_fFalloff,		0.53f );
		prm->set_float( p_fWaterColourR,	0.12f );
		prm->set_float( p_fWaterColourG,	0.22f );
		prm->set_float( p_fWaterColourB,	0.29f );
		prm->set_float(	p_fLODbias,		10.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		false );
		prm->set_bool(	p_bDrawIsland,		false );
		prm->set_bool(	p_bDiffuseRefl,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.36f );
		prm->set_float( p_fSunShininess,	1263.0f );
		prm->set_float( p_fSunStrength,		208.0f );
		break;
	case 4:
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	4.0f );
		prm->set_float( p_fFalloff,		0.47f );
		prm->set_float( p_fWaterColourR,	0.13f );
		prm->set_float( p_fWaterColourG,	0.19f );
		prm->set_float( p_fWaterColourB,	0.22f );
		prm->set_float(	p_fLODbias,		4.5f);
		prm->set_bool(	p_bAsPoints,		true );
		prm->set_bool(	p_bDrawDuckie,		false );
		prm->set_bool(	p_bDrawIsland,		false );
		prm->set_bool(	p_bDiffuseRefl,		true );
		break;
	case 5:
		prm->set_float( p_fScale,		0.197f );
		prm->set_float( p_fStrength,	12.9f );
		prm->set_float( p_fFalloff,		0.467f );
		prm->set_float( p_fWaterColourR,	0.12f );
		prm->set_float( p_fWaterColourG,	0.20f );
		prm->set_float( p_fWaterColourB,	0.24f );
		prm->set_float(	p_fLODbias,			0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.88f );
		prm->set_float( p_fSunShininess,	1263.0f );
		prm->set_float( p_fSunStrength,		5270.0f );
		prm->set_bool(	p_bDiffuseRefl,		true );
		break;
	case 6:
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	11.3f );
		prm->set_float( p_fFalloff,		0.56f );
		prm->set_float( p_fWaterColourR,	0.17f );
		prm->set_float( p_fWaterColourG,	0.27f );
		prm->set_float( p_fWaterColourB,	0.26f );
		prm->set_float(	p_fLODbias,			0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.256f );
		prm->set_float( p_fSunShininess,	700.0f );
		prm->set_float( p_fSunStrength,		200.85f );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	}
}

//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{
	if( g_pd3dDevice != NULL )
		g_pd3dDevice->Release();

	if( g_pD3D != NULL )
		g_pD3D->Release();
}



//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets m_vecUp the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws everything
//-----------------------------------------------------------------------------
void Render()
{	
/*	static DWORD it=0;
	static DWORD time_prepare, time_total, time_total_start;
	it++;
	// measure fps
	if (it>63)
	{
		time_total = timeGetTime() - time_total_start;
		time_total_start = timeGetTime();
		it = 0;
	}

	// update mouse & kbd
	mouse->Update();
	float movespeed;
	if( GetAsyncKeyState(VK_LCONTROL) != 0)
	{
		mspeed = 0.0001;
		movespeed = 0.03f;
	} 
	else if( GetAsyncKeyState(VK_LSHIFT) != 0)
	{
		mspeed = 0.05;
		movespeed = 5.0f;
	} 
	else
	{
		mspeed = 0.005;
		movespeed = 0.3f;
	}

	// keys

	if(keys['A'])
		camera_mouse->m_vecPos -= movespeed*camera_mouse->m_vecRight;
	if(keys['D'])
		camera_mouse->m_vecPos += movespeed*camera_mouse->m_vecRight;
	if(keys['W'])
		camera_mouse->m_vecPos += movespeed*camera_mouse->m_vecLook;
	if(keys['S'])
		camera_mouse->m_vecPos -= movespeed*camera_mouse->m_vecLook;
	if(keys['Q'])
		camera_mouse->m_vecPos += movespeed*camera_mouse->m_vecUp;
	if(keys['Z'])
		camera_mouse->m_vecPos -= movespeed*camera_mouse->m_vecUp;
	camera_mouse->update();

	if(mouse->mousedown(MOUSE_LEFT)){
		camera_mouse->roty -= 0.005 * mouse->x;
		camera_mouse->rotx -= 0.005 * mouse->y;
		camera_mouse->update();
	} else if(mouse->mousedown(MOUSE_RIGHT)){
		camera_mouse->m_vecPos += 5 * mspeed * mouse->x * camera_mouse->m_vecRight;
		camera_mouse->m_vecPos -= 5 * mspeed * mouse->y * camera_mouse->m_vecUp;
		camera_mouse->update();
	} else if(mouse->mousedown(MOUSE_MIDDLE)){
		camera_mouse->m_vecPos -= 10*mspeed * mouse->y * camera_mouse->m_vecLook;
		//camera_mouse->rotz -= mspeed * mouse->x;		
		camera_mouse->update();
	} 
*/
	if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
	{	
		
/*	// set lightning
	sun.Direction.x = -cos(prm->get_float(p_fSunPosTheta))*sin(prm->get_float(p_fSunPosAlpha));
	sun.Direction.y = -sin(prm->get_float(p_fSunPosTheta));
	sun.Direction.z = -cos(prm->get_float(p_fSunPosTheta))*cos(prm->get_float(p_fSunPosAlpha));
	//if(mode==rsm_reflection)
	{
		//sun.Direction.x = -sun.Direction.x;
		sun.Direction.y = -sun.Direction.y;
		//sun.Direction.z = -sun.Direction.z;
	}

	sun.Diffuse.r = 2.0f;
	sun.Diffuse.g = 2.0f;
	sun.Diffuse.b = 2.0f;
	sun.Diffuse.a = 1.0f;
	sun.Ambient.a = 1.0f;
	sun.Ambient.r = 0.2f;
	sun.Ambient.g = 0.3f;
	sun.Ambient.b = 0.3f;
	sun.Specular.r = 1.0f;
	sun.Specular.g = 1.0f;
	sun.Specular.b = 1.0f;
	sun.Specular.a = 1.0f;

	sun.Attenuation0 = 1.0f;
	sun.Type = D3DLIGHT_DIRECTIONAL;
	g_pd3dDevice->SetLight(0, &sun);
	g_pd3dDevice->LightEnable( 0, true);	
*/		
	water.Rander();
			
/*

		// Draw text		
		if (text)
		{
			//		g_pd3dxFont->Begin();		

			RECT textpos;
			char textString[900],activecamera[16],view[16];
			SetRect( &textpos, 5, 5, 0, 0 );

			if(camera_mouse==cameras[0])
				sprintf(activecamera,"tesselating");
			else if(camera_mouse==cameras[1])
				sprintf(activecamera,"observing");

			if(camera_view==cameras[0])
				sprintf(view,"primary");
			else if(camera_view==cameras[1])
				sprintf(view,"observer");

			sprintf( textString, "Mouse is currently controlling the %s camera (switch with TAB)\n",activecamera);
			sprintf( textString, "%sefficiency: %.2f percent  \n",textString,100*sea->efficiency);
			sprintf( textString, "%sdetail: %ix%i\n",textString,gridsize_x,gridsize_y);
			sprintf( textString, "%sview(switch with 'c'): %s\n",textString,view);
			//sprintf( textString, "%s\n%s",textString, sea->debugdata);
			sprintf( textString, "%stime_prepare: %i\ntotal time: %f\nframerate: %f Hz\n",textString,time_prepare, (double)time_total/64.0f,64000.0f/((float)(time_total)));
			//		sprintf( textString, "%sn_points: %i\n",textString,n_points);
			//		sprintf( textString, "%sx_min: %f x_max: %f y_min: %f y_max: %f ",textString,x_min,x_max,y_min,y_max);

			g_pd3dxFont->DrawText(NULL, textString,-1,&textpos,DT_NOCLIP,D3DXCOLOR( 0.0f, 0.0f, 0.0f, 1.0f ));

			sprintf( textString, "%s\n", prm->get_display() );
			textpos.top = 200;
			g_pd3dxFont->DrawText(NULL, textString,-1,&textpos,DT_NOCLIP,D3DXCOLOR( 1.0f, 0.5f, 0.5f, 1.0f ));
			//		g_pd3dxFont->End();
		}
*/
		// End the scene
		g_pd3dDevice->EndScene();
	}
	// Present the backbuffer contents to the display

	//LPDIRECT3DSWAPCHAIN9 SwapChain;
	//g_pd3dDevice->GetSwapChain( 0,&SwapChain);
	//SwapChain->Present( NULL, NULL, NULL, NULL, D3DPRESENT_LINEAR_CONTENT );			

	g_pd3dDevice->Present(NULL, NULL, NULL, NULL );	
}



//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_DESTROY:
		Cleanup();
		PostQuitMessage( 0 );
		return 0;
	case WM_KEYDOWN:							// Is A Key Being Held Down?
		{			
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			// preset handling
			if (keys['1'])
			{
				loadpreset(1);
				return 0;
			}
			if (keys['2'])
			{
				loadpreset(2);
				return 0;
			}
			if (keys['3'])
			{
				loadpreset(3);
				return 0;
			}
			if (keys['4'])
			{
				loadpreset(4);
				return 0;
			}
			if (keys['5'])
			{
				loadpreset(5);
				return 0;
			}
			if (keys['6'])
			{
				loadpreset(6);
				return 0;
			}
			if (keys['T'])
			{
				text = !text;
				return 0;
			}
			// other keys
			if (keys[VK_TAB])
			{
				active_camera_mouse = (active_camera_mouse+1)%n_cameras;
				camera_mouse = cameras[active_camera_mouse];
				return 0;
			}
			if (keys['C'])
			{
				active_camera_view = (active_camera_view+1)%n_cameras;
				camera_view = cameras[active_camera_view];
				return 0;
			} 
			if (keys['E'])
			{
				sea->calc_efficiency();
				return 0;
			} 
			if (keys['0']) 
			{
				prm->value_reset();
				return 0;
			}

			if (keys[VK_PRIOR]){
				gridsize_x = gridsize_x<<1;
				sea->set_grid_size(gridsize_x,gridsize_y);
				return 0;
			}
			if (keys[VK_NEXT]){
				gridsize_x = gridsize_x>>1;
				if (gridsize_x<2) gridsize_x = 2;
				sea->set_grid_size(gridsize_x,gridsize_y);
				return 0;
			}
			if (keys[VK_HOME]){
				gridsize_y = gridsize_y<<1;
				sea->set_grid_size(gridsize_x,gridsize_y);
				return 0;
			}
			if (keys[VK_END]){
				gridsize_y = gridsize_y>>1;
				if (gridsize_y<2) gridsize_y = 2;
				sea->set_grid_size(gridsize_x,gridsize_y);
				return 0;
			}
			if (keys[VK_SUBTRACT])
			{
				prm->value_decrease();
				return 0;
			}
			if (keys[VK_ADD])
			{
				prm->value_increase();				
				return 0;
			}

			if (keys[VK_LEFT])
			{
				prm->value_decreaseXL();
				return 0;
			}
			if (keys[VK_RIGHT])
			{
				prm->value_increaseXL();
				return 0;
			}
			if (keys[VK_UP])
			{
				prm->previous_parameter();
				return 0;
			}
			if (keys[VK_DOWN])
			{
				prm->next_parameter();
				return 0;
			}			

			return 0;								// Jump Back
		}

	case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

	}

	return DefWindowProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
	// Register the window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
		GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
		L"D3D Tutorial", NULL };
	RegisterClassEx( &wc );

	// Create the application's window
	HWND hWnd = CreateWindow( L"D3D Tutorial", L"Projected grid concept demo",
		WS_OVERLAPPEDWINDOW, 100, 100, 1024, 768,
		GetDesktopWindow(), NULL, wc.hInstance, NULL );
	// init mouse
	mouse = new dxmouse(hWnd,hInst);
	mouse->Init();

	// Initialize Direct3D
	if( SUCCEEDED( InitD3D( hWnd ) ) )
	{
		// Create the scene geometry
		// Show the window
		ShowWindow( hWnd, SW_SHOWDEFAULT );
		UpdateWindow( hWnd );

		// Enter the message loop
		MSG msg;
		ZeroMemory( &msg, sizeof(msg) );
		while( msg.message!=WM_QUIT )
		{
			if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
			else
				Render();
		}
	}

	UnregisterClass( L"D3D Tutorial", wc.hInstance );
	return 0;
}


/*
//-----------------------------------------------------------------------------
// Name: createFont()
// Desc: 
//-----------------------------------------------------------------------------
void createFont( void )
{
	//
	// To create a Windows friendly font using only a point size, an 
	// application must calculate the logical height of the font.
	// 
	// This is because functions like CreateFont() and CreateFontIndirect() 
	// only use logical units to specify height.
	//
	// Here's the formula to find the height in logical pixels:
	//
	//             -( point_size * LOGPIXELSY )
	//    height = ----------------------------
	//                          72
	//

	HRESULT hr;
	HDC hDC;
	HFONT hFont;
	int nHeight;
	int nPointSize = 7;
	char strFontName[] = "Verdana";

	hDC = GetDC( NULL );

	nHeight = -( MulDiv( nPointSize, GetDeviceCaps(hDC, LOGPIXELSY), 72 ) );


	hFont = CreateFont( nHeight, 
		0, 0, 0,
		FW_DONTCARE,
		false, false, false,
		DEFAULT_CHARSET,
		OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH | FF_DONTCARE,
		strFontName );

	if( hFont != NULL )
	{

		//if( FAILED( D3DXCreateFont( g_pd3dDevice, hFont, &g_pd3dxFont ) ) )
		if( FAILED( D3DXCreateFont( g_pd3dDevice,nHeight, 5, 1, 1, false, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, strFontName, &g_pd3dxFont ) ) )
		{
			MessageBox(NULL,L"Call to D3DXCreateFont failed!", L"ERROR",MB_OK|MB_ICONEXCLAMATION);
		}


		DeleteObject( hFont );
	}
	else
	{
		MessageBox(NULL,L"Call to CreateFont failed!", L"ERROR",MB_OK|MB_ICONEXCLAMATION);
	}

}
*/
